@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -57,7 +57,7 @@ use Encode ();
 use File::Spec ();
 use Cwd ();
 use Scalar::Util qw(blessed);
-use UNIVERSAL::require;
+use RT::StaticUtil;
 
 use vars qw($Config $System $SystemUser $Nobody $Handle $Logger $CurrentInterface $_Privileged $_Unprivileged $_INSTALL_MODE);
 
@@ -508,6 +508,8 @@ sub InitClasses {
     require RT::Configurations;
     require RT::REST2;
     require RT::Authen::Token;
+    require RT::Shortener;
+    require RT::Shorteners;
 
     _BuildTableAttributes();
 
@@ -522,7 +524,7 @@ sub InitClasses {
         }
 
         foreach my $class ( grep $_, RT->Config->Get('CustomFieldValuesSources') ) {
-            $class->require or $RT::Logger->error(
+            RT::StaticUtil::RequireModule($class) or $RT::Logger->error(
                 "Class '$class' is listed in CustomFieldValuesSources option"
                 ." in the config, but we failed to load it:\n$@\n"
             );
@@ -569,6 +571,7 @@ sub _BuildTableAttributes {
         RT::Catalog
         RT::CustomRole
         RT::ObjectCustomRole
+        RT::Shortener
     );
 }
 
@@ -846,8 +849,8 @@ sub InitPlugins {
         if ( $CORED_PLUGINS{$plugin} ) {
             RT->Logger->warning( "$plugin has been cored since RT $CORED_PLUGINS{$plugin}, please check the upgrade document for more details" );
         }
-        $plugin->require;
-        die $UNIVERSAL::require::ERROR if ($UNIVERSAL::require::ERROR);
+        my ($success, $message) = RT::StaticUtil::RequireModule($plugin);
+        die $message unless $success;
         push @plugins, RT::Plugin->new(name =>$plugin);
     }
     return @plugins;
