@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -203,6 +203,7 @@ A complete list of acceptable fields:
     *  Queue           => Name or id# of a queue
        Subject         => A text string
      ! Status          => A valid status. Defaults to 'new'
+       SLA             => A valid SLA level
        Due             => Dates can be specified in seconds since the epoch
                           to be handled literally or in a semi-free textual
                           format which RT will attempt to parse.
@@ -765,6 +766,7 @@ sub ParseLines {
         SquelchMailTo   => $args{'squelchmailto'},
         Type            => $args{'type'},
         SkipCreate      => $args{'skipcreate'} || 0,
+        SLA             => $args{'sla'},
     );
 
     if ( $args{content} ) {
@@ -1156,7 +1158,7 @@ sub UpdateCustomFields {
         $CustomFieldObj->LoadById($cf);
 
         my @values;
-        if ($CustomFieldObj->Type =~ /text/i) { # Both Text and Wikitext
+        if ($CustomFieldObj->Type =~ /text|html/i) { # Text, HTML, and Wikitext
             @values = ($args->{$arg});
         } else {
             @values = split /\n/, $args->{$arg};
