@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -303,7 +303,8 @@ sub _SerializeContent {
 
 sub SetContent {
     my $self = shift;
-    my $content = shift;
+    my %args = ( Content => undef, SyncLinks => 1, @_ % 2 ? ( Content => @_ ) : @_ );
+    my $content = $args{Content};
 
     # Call __Value to avoid ACL check.
     if ( ($self->__Value('ContentType')||'') eq 'storable' ) {
@@ -314,9 +315,10 @@ sub SetContent {
             return(0, "Content couldn't be frozen");
         }
     }
-    my ($ok, $msg) = $self->_Set( Field => 'Content', Value => $content );
+    my ( $ok, $msg )
+        = $self->_Set( Field => 'Content', Value => $content, RecordTransaction => $args{RecordTransaction} );
     if ($ok) {
-        $self->_SyncLinks;
+        $self->_SyncLinks if $args{SyncLinks};
         return ( $ok, $self->loc("Attribute updated") );
     }
     return ($ok, $msg);
@@ -457,6 +459,17 @@ sub Delete {
                 RT->Logger->error( "Couldn't delete link #" . $link->id . ": $msg" );
             }
         }
+
+        if ( $name eq 'SavedSearch' ) {
+            my $shortener = RT::Shortener->new( $self->CurrentUser );
+            $shortener->LoadByContent( 'SavedSearchId=' . $self->Id );
+            if ( $shortener->Id ) {
+                my ( $ret, $msg ) = $shortener->Delete;
+                if ( !$ret ) {
+                    RT->Logger->error( "Couldn't delete shortener #" . $shortener->Id . ": $msg" );
+                }
+            }
+        }
     }
 
     return @return;
@@ -861,11 +874,72 @@ sub PreInflate {
     if ($data->{Object} and ref $data->{Object}) {
         my $on_uid = ${ $data->{Object} };
 
+        my $force;
+
         # skip attributes of objects we're not inflating
-        # exception: we don't inflate RT->System, but we want RT->System's searches
-        unless ($on_uid eq RT->System->UID && $data->{Name} =~ /Search/) {
-            return if $importer->ShouldSkipTransaction($on_uid);
+        if ( $on_uid eq RT->System->UID ) {
+
+            # We always want RT->System's searches and dashboards
+            $force = 1 if $data->{Name} =~ /^Search|^(?:SavedSearch|Dashboard|ContentHistory)$/;
+
+            # Do not import DefaultDashboard if it already exists
+            if ( $data->{Name} eq 'DefaultDashboard' ) {
+                if ( my $exists = RT->System->FirstAttribute('DefaultDashboard') ) {
+                    $importer->Resolve( $uid => ref($exists) => $exists->Id );
+                    return;
+                }
+                else {
+                    $force = 1;
+                }
+            }
         }
+        elsif ( $on_uid =~ /^RT::(?:User|Group)-/ ) {
+            if ( $importer->ShouldSkipTransaction($on_uid) ) {
+                if ( $data->{Name} eq 'Bookmarks' || $data->{Name} =~ /^Pref-/ ) {
+                    my $obj = $importer->LookupObj($on_uid);
+                    if ( $obj && $obj->Id ) {
+                        if ( my $exists = $obj->FirstAttribute( $data->{Name} ) ) {
+                            if ( $data->{Name} eq 'Bookmarks' ) {
+
+                                # Merge bookmarks if possible
+                                my $new_content = $exists->_DeserializeContent( $data->{Content} );
+                                if ( ref $new_content eq 'ARRAY' ) {
+                                    my $content = $exists->Content;
+                                    my $changed;
+                                    for my $uid (@$new_content) {
+                                        if ( my $ticket = $importer->LookupObj($$uid) ) {
+                                            $content->{ $ticket->Id } = 1;
+                                            $changed = 1;
+                                        }
+                                    }
+                                    if ($changed) {
+                                        my ( $ret, $msg ) = $exists->SetContent($content);
+                                        unless ($ret) {
+                                            RT->Logger->error("Couldn't update Bookmarks for user $on_uid: $msg");
+                                        }
+                                    }
+                                }
+                            }
+                            $importer->Resolve( $uid => ref($exists) => $exists->Id );
+                        }
+                        else {
+                            $force = 1;
+                        }
+                    }
+                }
+                elsif ( $data->{Name} =~ /SavedSearch|Dashboard|Subscription|ContentHistory/ ) {
+
+                    # We always want saved searches and dashboards
+                    $force = 1;
+                }
+            }
+            elsif ( $data->{Name} eq 'RecentlyViewedTickets' ) {
+                # Don't bother importing frequently updated recently viewed tickets
+                return;
+            }
+        }
+
+        return if !$force && $importer->ShouldSkipTransaction($on_uid);
     }
 
     return $class->SUPER::PreInflate( $importer, $uid, $data );
@@ -909,12 +983,12 @@ sub PostInflateFixup {
                 }
             }
         }
-        $self->SetContent($content);
+        $self->SetContent( $content, SyncLinks => 0, RecordTransaction => 0 );
     }
     elsif ( $self->Name =~ /DefaultDashboard$/ ) {
         my $content = $self->Content;
         if ( ref($content) eq 'SCALAR' ) {
-            my $attr = $importer->LookupObj($$_);
+            my $attr = $importer->LookupObj($$content);
             if ($attr) {
                 $content = $attr->Id;
             }
@@ -953,7 +1027,7 @@ sub PostInflateFixup {
                 }
             }
         }
-        $self->SetContent($content);
+        $self->SetContent( $content, SyncLinks => 0, RecordTransaction => 0 );
     }
     elsif ($self->Name eq 'Subscription') {
         my $content = $self->Content;
@@ -970,7 +1044,7 @@ sub PostInflateFixup {
                 );
             }
         }
-        $self->SetContent($content);
+        $self->SetContent( $content, SyncLinks => 0, RecordTransaction => 0 );
     }
 }
 
