@@ -3,7 +3,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -300,11 +300,14 @@ sub _Create {
         Description => undef,
         Domain      => undef,
         Instance    => '0',
+        Disabled    => 0,
         InsideTransaction => undef,
         _RecordTransaction => 1,
         @_
     );
 
+    $args{'Name'} = $self->CanonicalizeName( $args{'Name'} );
+
     if ($args{'Domain'}) {
         # Enforce uniqueness on user defined group names
         if ($args{'Domain'} eq 'UserDefined') {
@@ -325,6 +328,7 @@ sub _Create {
     my $principal    = RT::Principal->new( $self->CurrentUser );
     my $principal_id = $principal->Create(
         PrincipalType => 'Group',
+        Disabled      => $args{'Disabled'} // 0,
     );
 
     $self->SUPER::Create(
@@ -647,7 +651,7 @@ sub SetName {
     my $self = shift;
     my $value = shift;
 
-    my ($status, $msg) = $self->_Set( Field => 'Name', Value => $value );
+    my ($status, $msg) = $self->_Set( Field => 'Name', Value => $self->CanonicalizeName($value) );
     return ($status, $msg);
 }
 
@@ -1070,7 +1074,7 @@ sub _AddMember {
     }
 
     return (1, $self->loc("[_1] set to [_2]",
-                          $self->loc($self->Name), $new_member_obj->Object->Name) )
+                          $self->Label, $new_member_obj->Object->Name) )
         if $self->SingleMemberRoleGroup;
 
     return ( 1, $self->loc("Member added: [_1]", $new_member_obj->Object->Name) );
@@ -1731,6 +1735,11 @@ sub PreInflate {
         $queue->Load( $data->{Instance} );
         $obj->LoadRoleGroup( Object => $queue, Name => $data->{Name} );
         return $duplicated->() if $obj->Id;
+    } elsif ($data->{Domain} eq 'RT::Catalog-Role') {
+        my $catalog = RT::Catalog->new( RT->SystemUser );
+        $catalog->Load( $data->{Instance} );
+        $obj->LoadRoleGroup( Object => $catalog, Name => $data->{Name} );
+        return $duplicated->() if $obj->Id;
     }
 
     my $principal = RT::Principal->new( RT->SystemUser );
@@ -1790,6 +1799,20 @@ sub URI {
     return $uri->URIForObject($self);
 }
 
+=head2 CanonicalizeName NAME
+
+Strip leading/trailing spaces and returns the updated name.
+
+=cut
+
+sub CanonicalizeName {
+    my $self = shift;
+    my $name = shift // return undef;
+    $name =~ s!^\s+!!;
+    $name =~ s!\s+$!!;
+    return $name;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
