@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -160,7 +160,7 @@ sub Load {
     }
 
     my $class = "RT::Lifecycle::".ucfirst($args{Type});
-    bless $self, $class if $class->require;
+    bless $self, $class if RT::StaticUtil::RequireModule($class);
 
     return $self;
 }
@@ -764,7 +764,7 @@ sub FillCache {
         }
 
         my $class = "RT::Lifecycle::".ucfirst($type);
-        $class->RegisterRights if $class->require
+        $class->RegisterRights if RT::StaticUtil::RequireModule($class)
             and $class->can("RegisterRights");
     }
 
@@ -1045,10 +1045,10 @@ sub ValidateLifecycle {
             push @warnings, $current_user->loc( "Invalid action status change [_1], in [_2] lifecycle", $transition, $name );
             next;
         }
-        push @warnings, $current_user->loc( "Nonexistant status [_1] in action in [_2] lifecycle", lc $from, $name )
+        push @warnings, $current_user->loc( "Nonexistant status [_1] in actions in [_2] lifecycle", lc $from, $name )
             unless $from eq '*'
             or $lifecycle->{canonical_case}{ lc $from };
-        push @warnings, $current_user->loc( "Nonexistant status [_1] in action in [_2] lifecycle", lc $to, $name )
+        push @warnings, $current_user->loc( "Nonexistant status [_1] in actions in [_2] lifecycle", lc $to, $name )
             unless $to eq '*'
             or $lifecycle->{canonical_case}{ lc $to };
     }
@@ -1084,6 +1084,10 @@ sub ValidateLifecycleMaps {
         push @warnings, $current_user->loc( "Nonexistant lifecycle [_1] in [_2] lifecycle map", $to, $mapname )
             unless $LIFECYCLES_CACHE{$to};
 
+        # Ignore mappings referring to disabled lifecycles
+        next if $LIFECYCLES_CACHE{$from} && $LIFECYCLES_CACHE{$from}{disabled};
+        next if $LIFECYCLES_CACHE{$to} && $LIFECYCLES_CACHE{$to}{disabled};
+
         my $map = $LIFECYCLES_CACHE{'__maps__'}{$mapname};
         for my $status ( keys %{$map} ) {
             push @warnings, $current_user->loc( "Nonexistant status [_1] in [_2] in [_3] lifecycle map", lc $status, $from, $mapname )
