@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -481,6 +481,7 @@ sub CustomRoles {
     my $roles = RT::CustomRoles->new( $self->CurrentUser );
     if ( $self->CurrentUserHasRight('SeeQueue') ) {
         $roles->LimitToObjectId( $self->Id );
+        $roles->LimitToLookupType(RT::Ticket->CustomFieldLookupType);
         $roles->ApplySortOrder;
     }
     else {
@@ -664,19 +665,17 @@ sub AdminCc {
 }
 
 
-
-# a generic routine to be called by IsRequestor, IsCc and IsAdminCc
+# a generic routine to be called by IsCc and IsAdminCc
 
 =head2 IsWatcher { Type => TYPE, PrincipalId => PRINCIPAL_ID }
 
-Takes a param hash with the attributes Type and PrincipalId
+Takes a param hash with the attributes Type and PrincipalId.
 
-Type is one of Requestor, Cc, AdminCc and Owner
+Type is one of Cc or AdminCc.
 
-PrincipalId is an RT::Principal id 
-
-Returns true if that principal is a member of the group Type for this queue
+PrincipalId is an RT::Principal id.
 
+Returns true if that principal is a member of the group Type for this queue.
 
 =cut
 
@@ -702,8 +701,6 @@ sub IsWatcher {
 }
 
 
-
-
 =head2 IsCc PRINCIPAL_ID
 
 Takes an RT::Principal id.
@@ -789,16 +786,7 @@ sub _Set {
     return ( $ret, $msg );
 }
 
-sub Lifecycle {
-    my $self        = shift;
-    my $context_obj = shift;
 
-    if ( $context_obj && $context_obj->QueueObj->Id eq $self->Id && $context_obj->CurrentUserHasRight('SeeQueue') ) {
-        return ( $self->__Value('Lifecycle') );
-    }
-
-    return ( $self->_Value('Lifecycle') );
-}
 
 sub _Value {
     my $self = shift;
@@ -903,13 +891,9 @@ Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
 =cut
 
 
-=head2 Lifecycle [CONTEXT_OBJ]
+=head2 Lifecycle
 
 Returns the current value of Lifecycle.
-
-Provide an optional ticket object as context to check role-level rights
-in addition to queue-level rights for SeeQueue.
-
 (In the database, Lifecycle is stored as varchar(32).)
 
 
@@ -1103,6 +1087,7 @@ sub FindDependencies {
     # Object Custom Roles
     $objs = RT::ObjectCustomRoles->new( $self->CurrentUser );
     $objs->LimitToObjectId($self->Id);
+    $objs->LimitToLookupType(RT::Ticket->CustomFieldLookupType);
     $deps->Add( in => $objs );
 }
 
@@ -1144,6 +1129,7 @@ sub __DependsOn {
 
 # Object Custom Roles
     $objs = RT::ObjectCustomRoles->new( $self->CurrentUser );
+    $objs->LimitToLookupType( RT::Ticket->CustomFieldLookupType );
     $objs->LimitToObjectId($self->Id);
     push( @$list, $objs );
 
@@ -1175,8 +1161,32 @@ sub DefaultValue {
     my $self = shift;
     my $field = shift;
     my $attr = $self->FirstAttribute('DefaultValues');
-    return undef unless $attr && $attr->Content;
-    return $attr->Content->{$field};
+
+    my $fallback;
+    if ( $field =~ /Priority/
+        && RT->Config->Get('EnablePriorityAsString') )
+    {
+        my %config = RT->Config->Get('PriorityAsString');
+        my $queue_name = $self->__Value('Name');
+        if ( my $value = exists $config{$queue_name} ? $config{$queue_name} : $config{Default} ) {
+
+            # For ordered list(array), use the first entry. For unordered list(hash), use the lowest value.
+            # This is also consistent with the priority order on web pages
+            if ( ref $value eq 'ARRAY' ) {
+                $fallback = $value->[1];
+            }
+            elsif ( ref $value eq 'HASH' ) {
+                $fallback = ( sort { $a <=> $b } values %$value )[0];
+            }
+            else {
+                RT->Logger->warning("Invalid PriorityAsString value: $value");
+            }
+        }
+    }
+
+    return $fallback unless $attr && $attr->Content;
+    my $value = $attr->Content->{$field};
+    return $value // $fallback;
 }
 
 sub SetDefaultValue {
