@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -1305,7 +1305,7 @@ You can use additional fields parameters to expand child blocks, for
 example (line wrapping inserted for readability):
 
     XX_RT_URL_XX/REST/2.0/tickets
-      ?fields=Owner,Status,Created,Subject,Queue,CustomFields,Requestor,Cc,AdminCc,RT::CustomRole-1
+      ?fields=Owner,Status,Created,Subject,Queue,CustomFields,Requestor,Cc,AdminCc,CustomRoles
       &fields[Queue]=Name,Description
 
 Says that in the result set for tickets, the extra fields for Owner, Status,
@@ -1360,13 +1360,13 @@ ticket is displayed in this example):
             }
          ],
          "AdminCc" : [],
-         "RT::CustomRole-1" : [
-            {
+         "CustomRoles" : {
+            "Manager": {
                "_url" : "XX_RT_URL_XX/REST/2.0/user/foo@example.com",
                "type" : "user",
                "id" : "foo@example.com"
             }
-         ]
+         }
       }
       { … },
       …
@@ -1449,17 +1449,13 @@ handle them appropriately.
 sub to_psgi_app {
     my $self = shift;
     my $res = $self->to_app(@_);
-
-    return Plack::Util::response_cb($res, sub {
-        my $res = shift;
-        $self->CleanupRequest;
-    });
 }
 
 sub to_app {
     my $class = shift;
 
     return builder {
+        enable '+RT::REST2::Middleware::CleanupRequest';
         enable '+RT::REST2::Middleware::ErrorAsJSON';
         enable '+RT::REST2::Middleware::Log';
         enable '+RT::REST2::Middleware::Auth';
@@ -1484,22 +1480,4 @@ sub PSGIWrap {
     };
 }
 
-sub CleanupRequest {
-
-    if ( $RT::Handle && $RT::Handle->TransactionDepth ) {
-        $RT::Handle->ForceRollback;
-        $RT::Logger->crit(
-            "Transaction not committed. Usually indicates a software fault."
-            . "Data loss may have occurred" );
-    }
-
-    # Clean out the ACL cache. the performance impact should be marginal.
-    # Consistency is imprived, too.
-    RT::Principal->InvalidateACLCache();
-    DBIx::SearchBuilder::Record::Cachable->FlushCache
-      if ( RT->Config->Get('WebFlushDbCacheEveryRequest')
-        and UNIVERSAL::can(
-            'DBIx::SearchBuilder::Record::Cachable' => 'FlushCache' ) );
-}
-
 1;
