@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -180,7 +180,7 @@ sub LoadAction  {
 
     my $module = $self->ExecModule;
     my $type = 'RT::Action::' . $module;
-    $type->require or die "Require of $type action module failed.\n$@\n";
+    RT::StaticUtil::RequireModule($type) or die "Require of $type action module failed.\n$@\n";
 
     return $self->{'Action'} = $type->new(
         %args,
