@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -176,7 +176,7 @@ sub LoadCondition  {
 
     my $module = $self->ExecModule;
     my $type = 'RT::Condition::' . $module;
-    $type->require or die "Require of $type condition module failed.\n$@\n";
+    RT::StaticUtil::RequireModule($type) or die "Require of $type condition module failed.\n$@\n";
 
     return $self->{'Condition'} = $type->new(
         ScripConditionObj => $self,
