@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -313,6 +313,7 @@ Set( \$WebPath,   "");
 Set( \@LexiconLanguages, qw(en zh_TW zh_CN fr ja));
 Set( \$RTAddressRegexp , qr/^bad_re_that_doesnt_match\$/i);
 Set( \$ShowHistory, "always");
+Set( \$WebSecureCookies, 0);
 };
     if ( $ENV{'RT_TEST_DB_SID'} ) { # oracle case
         print $config "Set( \$DatabaseName , '$ENV{'RT_TEST_DB_SID'}' );\n";
@@ -1535,8 +1536,36 @@ sub trust_gnupg_key {
     );
 }
 
+=head2 started_ok
+
+Starts the test web server, referencing $ENV{'RT_TEST_WEB_HANDLER'}
+to determine which web server to use. Defaults to an inline server
+running with Plack.
+
+Returns the base URL of the started server, including the port.
+Also returns a Mechanize object useful for additional tests.
+
+    my ( $baseurl, $m ) = RT::Test->started_ok();
+    diag "Test server running at: $baseurl";
+    ok( $m->login, 'Log in' );
+
+If your tests change configuration stored in the DB as part of
+the test, you can disable the DB config cache:
+
+    my ( $baseurl, $m ) = RT::Test->started_ok( disable_config_cache => 1 );
+
+Without this, you may find tests intermittently failing because
+the config update code needs 1 second to recognize an update is
+needed.
+
+=cut
+
 sub started_ok {
     my $self = shift;
+    my %args = (
+                disable_config_cache => '0',
+                @_
+    );
 
     require RT::Test::Web;
 
@@ -1546,6 +1575,7 @@ sub started_ok {
            ."Pass server_ok => 1 if you know what you're doing.";
     }
 
+    $ENV{'RT_TEST_DISABLE_CONFIG_CACHE'} = 1 if $args{'disable_config_cache'};
 
     $ENV{'RT_TEST_WEB_HANDLER'} = undef
         if $rttest_opt{actual_server} && ($ENV{'RT_TEST_WEB_HANDLER'}||'') eq 'inline';
