@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -284,16 +284,30 @@ sub _EnumLimit {
     # SQL::Statement changes != to <>.  (Can we remove this now?)
     $op = "!=" if $op eq "<>";
 
-    die "Invalid Operation: $op for $field"
-        unless $op eq "="
-        or $op     eq "!=";
+    die "Invalid Operation: $op for $field" unless $op =~ /^(?:=|!=|IN|NOT IN)$/i;
 
     my $meta = $FIELD_METADATA{$field};
     if ( defined $meta->[1] && defined $value && $value !~ /^\d+$/ ) {
         my $class = "RT::" . $meta->[1];
-        my $o     = $class->new( $sb->CurrentUser );
-        $o->Load($value);
-        $value = $o->Id || 0;
+        if ( ref $value eq 'ARRAY' ) {
+            my @values;
+            for my $i (@$value) {
+                if ( $i !~ /^\d+$/ ) {
+                    my $o = $class->new( $sb->CurrentUser );
+                    $o->Load($i);
+                    push @values, $o->Id || 0;
+                }
+                else {
+                    push @values, $i;
+                }
+            }
+            $value = \@values;
+        }
+        else {
+            my $o = $class->new( $sb->CurrentUser );
+            $o->Load($value);
+            $value = $o->Id || 0;
+        }
     }
     $sb->Limit(
         FIELD    => $field,
@@ -807,15 +821,37 @@ sub _TicketLimit {
     $field =~ s!^Ticket!!;
 
     if ( $field eq 'Queue' && $value =~ /\D/ ) {
-        my $queue = RT::Queue->new($self->CurrentUser);
-        $queue->Load($value);
-        $value = $queue->id if $queue->id;
+        if ( ref $value eq 'ARRAY' ) {
+            my @values;
+            for my $v ( @$value ) {
+                my $o = RT::Queue->new( $self->CurrentUser );
+                $o->Load($v);
+                push @values, $o->Id || 0;
+            }
+            $value = \@values;
+        }
+        else {
+            my $queue = RT::Queue->new($self->CurrentUser);
+            $queue->Load($value);
+            $value = $queue->id if $queue->id;
+        }
     }
 
-    if ( $field =~ /^(?:Owner|Creator)$/ && $value =~ /\D/ ) {
-        my $user = RT::User->new( $self->CurrentUser );
-        $user->Load($value);
-        $value = $user->id if $user->id;
+    if ( $field =~ /^(?:Owner|Creator|LastUpdatedBy)$/ && $value =~ /\D/ ) {
+        if ( ref $value eq 'ARRAY' ) {
+            my @values;
+            for my $v ( @$value ) {
+                my $o = RT::User->new( $self->CurrentUser );
+                $o->Load($v);
+                push @values, $o->Id || 0;
+            }
+            $value = \@values;
+        }
+        else {
+            my $user = RT::User->new( $self->CurrentUser );
+            $user->Load($value);
+            $value = $user->id if $user->id;
+        }
     }
 
     $self->Limit(
@@ -896,18 +932,14 @@ sub _parser {
     );
     die join "; ", map { ref $_ eq 'ARRAY' ? $_->[ 0 ] : $_ } @results if @results;
 
+    my $queues = $tree->GetReferencedQueues( CurrentUser => $self->CurrentUser );
+    my %referenced_lifecycle = map { $_->{Lifecycle} => 1 } values %$queues;
+
     if ( RT->Config->Get('EnablePriorityAsString') ) {
-        my $queues = $tree->GetReferencedQueues( CurrentUser => $self->CurrentUser );
         my %config = RT->Config->Get('PriorityAsString');
         my @names;
         if (%$queues) {
-            for my $id ( keys %$queues ) {
-                my $queue = RT::Queue->new( $self->CurrentUser );
-                $queue->Load($id);
-                if ( $queue->Id ) {
-                    push @names, $queue->__Value('Name');    # Skip ACL check
-                }
-            }
+            @names = map { $_->{Name} } values %$queues;
         }
         else {
             @names = keys %config;
@@ -965,6 +997,7 @@ sub _parser {
                       map { $_ => $RT::Lifecycle::LIFECYCLES{ $_ }{ inactive } }
                       grep { @{ $RT::Lifecycle::LIFECYCLES{ $_ }{ inactive } || [] } }
                       grep { $RT::Lifecycle::LIFECYCLES_CACHE{ $_ }{ type } eq 'ticket' }
+                      grep { %referenced_lifecycle ? $referenced_lifecycle{$_} : 1 }
                       keys %RT::Lifecycle::LIFECYCLES;
                     return unless %lifecycle;
 
@@ -1008,6 +1041,7 @@ sub _parser {
                           || @{ $RT::Lifecycle::LIFECYCLES{ $_ }{ active }  || [] }
                       }
                       grep { $RT::Lifecycle::LIFECYCLES_CACHE{ $_ }{ type } eq 'ticket' }
+                      grep { %referenced_lifecycle ? $referenced_lifecycle{$_} : 1 }
                       keys %RT::Lifecycle::LIFECYCLES;
                     return unless %lifecycle;
 
@@ -1040,6 +1074,8 @@ sub _parser {
         }
     );
 
+    RT::SQL::_Optimize($tree);
+
     my $ea = '';
     $tree->traverse(
         sub {
@@ -1111,9 +1147,9 @@ sub _JoinTickets {
     unless ( defined $self->{_sql_aliases}{tickets} ) {
         $self->{_sql_aliases}{tickets} = $self->Join(
             TYPE   => 'LEFT',
-            FIELD1 => 'ObjectId',
             TABLE2 => 'Tickets',
             FIELD2 => 'id',
+            EXPRESSION => "main.ObjectId AND main.ObjectType='RT::Ticket'",
         );
     }
     return $self->{_sql_aliases}{tickets};
