@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -118,6 +118,8 @@ custom field.
                                      EmailAddress => 'mycroft@example.com',
                                      UserCF.Relationship => 'Brother' );
 
+If no Name is passed, it is set to the same value as EmailAddress.
+
 =cut
 
 
@@ -204,6 +206,13 @@ sub Create {
         return ( 0, $self->loc('Could not create user') );
     }
 
+    # Add corresponding CustomFieldValue records for custom fields,
+    # the same as in RT::Authen::ExternalAuth::UpdateUserInfo.
+    # RT::Authen::ExternalAuth needs Net::LDAP, which might be unavailable
+    if ( RT::StaticUtil::RequireModule("RT::Authen::ExternalAuth") ) {
+        RT::Authen::ExternalAuth::AddCustomFieldValue(%args);
+    }
+
     # Handle any user CFs
     $self->UpdateObjectCustomFieldValues( %args );
 
@@ -689,7 +698,6 @@ sub LoadOrCreateByEmail {
     return wantarray ? ($self->Id, $self->loc("User loaded")) : $self->Id
         if $self->Id;
 
-    $create{Name}       ||= $create{EmailAddress};
     $create{Privileged} ||= 0;
     $create{Comments}   //= 'Autocreated when added as a watcher';
 
@@ -834,9 +842,7 @@ is class name not an object.
 sub CanonicalizeEmailAddress {
     my $self = shift;
     my $email = shift;
-    # Example: the following rule would treat all email
-    # coming from a subdomain as coming from second level domain
-    # foo.com
+
     if ( my $match   = RT->Config->Get('CanonicalizeEmailAddressMatch') and
          my $replace = RT->Config->Get('CanonicalizeEmailAddressReplace') )
     {
