@@ -44,7 +44,8 @@ var showModal = function(html) {
 
     // We need to refresh the select picker plugin on AJAX calls
     // since the plugin only runs on page load.
-    jQuery('.selectpicker').selectpicker('refresh');
+    refreshSelectpicker();
+    RT.Autocomplete.bind(modal);
 };
 
 /* Classes */
@@ -365,18 +366,38 @@ function ReplaceAllTextareas() {
 
     for (var i=0; i < allTextAreas.length; i++) {
         var textArea = allTextAreas[i];
-        if (jQuery(textArea).hasClass("messagebox richtext")) {
+        if (jQuery(textArea).hasClass("richtext")) {
             // Turn the original plain text content into HTML
-            var type = jQuery("#"+textArea.name+"Type");
+            var type = jQuery('[name="'+textArea.name+'Type"]');
             if (type.val() != "text/html")
                 textArea.value = textToHTML(textArea.value);
 
             // Set the type
             type.val("text/html");
 
-            CKEDITOR.replace(textArea.name,{ width: '100%', height: RT.Config.MessageBoxRichTextHeight });
+            if (jQuery(textArea).hasClass("messagebox")) {
+                // * The "messagebox" class is used for ticket correspondence/comment content.
+                // * For a long time this was the only use of the CKEditor and it was given its own
+                //   user/system configuration option.
+                // * Continue using this config option for those CKEditor instances
+                CKEDITOR.replace(textArea.name,{ width: '100%', height: RT.Config.MessageBoxRichTextHeight });
+            }
+            else {
+                // * For all CKEditor instances without the "messagebox" class we instead base the
+                //   (editable) height on the size of the textarea element it's replacing.
+                //   The height does not include any toolbars, the status bar, or other "overhead".
+                // * The CKEditor box adds some additional padding around the edit area.
+                // * Specifically, in one browser/styling:
+                //   * there's 42px more top/bottom margin in the CKEditor than there is in the textarea
+                //   * the gap between lines is 3px taller in the CKEditor than it is in the textarea
+                //   + each new paragraph in the CKEditor adds an additional 13px to the gap between lines
+                //   So an adjustment of 54 px is added to create an area that will hold about 4/5
+                //   lines of text, similar to the plain text box. It will not scale the same for textareas
+                //   with different number of rows
+                CKEDITOR.replace(textArea.name,{ width: '100%', height: (jQuery(textArea).height() + 54) + 'px' });
+            }
 
-            jQuery("#" + textArea.name + "___Frame").addClass("richtext-editor");
+            jQuery('[name="' + textArea.name + '___Frame"]').addClass("richtext-editor");
         }
     }
 };
@@ -429,6 +450,7 @@ function AddAttachmentWarning() {
         var needsWarning = text &&
                            text.match(regex) &&
                            !dropzoneElement.hasClass('has-attachments') &&
+                           !jQuery('a.delete-attach').length &&
                            !has_reused_attachments;
 
         if (needsWarning) {
@@ -734,6 +756,27 @@ jQuery(function() {
     });
 
     if ( jQuery('.combobox').combobox ) {
+
+        // Override toggle so when user clicks the dropdown button, current value won't be cleared.
+        var orig_toggle = jQuery.fn.combobox.Constructor.prototype.toggle;
+        jQuery.fn.combobox.Constructor.prototype.toggle = function () {
+            if ( !this.disabled && !this.$container.hasClass('combobox-selected') && !this.shown && this.$element.val() ) {
+                // Show all the options
+                var matcher = this.matcher;
+                this.matcher = function () { return 1 };
+                this.lookup();
+                this.matcher = matcher;
+            }
+            else {
+                orig_toggle.apply(this);
+            }
+        };
+
+        // Trigger change event to update ValidationHint accordingly
+        jQuery.fn.combobox.Constructor.prototype.clearElement = function () {
+            this.$element.val('').change().focus();
+        };
+
         jQuery('.combobox').combobox({ clearIfNoMatch: false });
         jQuery('.combobox-wrapper').each( function() {
             jQuery(this).find('input[type=text]').prop('name', jQuery(this).data('name')).prop('value', jQuery(this).data('value'));
@@ -779,11 +822,12 @@ jQuery(function() {
     if ( RT.Config.WebDefaultStylesheet.match(/dark/) ) {
 
         // Add action type into iframe to customize default font color
-        jQuery(['action-response', 'action-private']).each(function(index, class_name) {
-            jQuery('.' + class_name).on('DOMNodeInserted', 'iframe', function(e) {
-                setTimeout(function() {
-                    jQuery(e.target).contents().find('.cke_editable').addClass(class_name);
-                }, 100);
+        CKEDITOR.on('instanceReady', function(e) {
+            var container = jQuery(e.editor.element.$.closest('div.messagebox-container'));
+            jQuery(['action-response', 'action-private']).each(function(index, class_name) {
+                if ( container.hasClass(class_name) ) {
+                    container.find('iframe').contents().find('.cke_editable').addClass(class_name);
+                }
             });
         });
 
@@ -862,8 +906,241 @@ jQuery(function() {
             file_input.prop('checked', false);
         });
     });
+
+    jQuery('form[name=BuildQuery] select[name^=SelectCustomField]').change(function() {
+        var form = jQuery(this).closest('form');
+        var row = jQuery(this).closest('div.form-row');
+        var val = jQuery(this).val();
+
+        var new_operator = form.find(':input[name="' + val + 'Op"]:first').clone();
+        row.children('div.operator').children().remove();
+        row.children('div.operator').append(new_operator);
+        row.children('div.operator').find('select.selectpicker').selectpicker();
+
+        var new_value = form.find(':input[name="ValueOf' + val + '"]:first');
+        if ( new_value.hasClass('ui-autocomplete-input') ) {
+            var source = new_value.autocomplete( "option" ).source;
+            new_value = new_value.clone();
+            new_value.autocomplete({ source: source });
+        }
+        else {
+            new_value = new_value.clone();
+        }
+
+        new_value.attr('id', null);
+        row.children('div.value').children().remove();
+        row.children('div.value').append(new_value);
+        row.children('div.value').find('select.selectpicker').selectpicker();
+        if ( new_value.hasClass('datepicker') ) {
+            new_value.removeClass('hasDatepicker');
+            initDatePicker(row);
+        }
+    });
+    jQuery(".search-filter").click(function(ev){
+        ev.preventDefault();
+        var modal = jQuery(this).closest('th').find('.modal.search-results-filter');
+        modal.css('top', jQuery(this).offset().top);
+        var left = jQuery(this).offset().left;
+        // 10 is extra space to move modal a bit away from edge
+        if ( left + modal.width() + 10 > jQuery('body').width() ) {
+            left = jQuery('body').width() - modal.width() - 10;
+        }
+        modal.css('left', left);
+        modal.find('div.modal-content').css('max-height', jQuery(window).height() - jQuery(this).offset().top - 10);
+        modal.modal('show');
+    });
+
+    jQuery('input[name=QueueChanged]').each(function() {
+        var form = jQuery(this).closest('form');
+        var mark_changed = function(name) {
+            if ( !form.find('input[name=ChangedField][value="' + name +'"]').length ) {
+                jQuery('<input type="hidden" name="ChangedField" value="' + name + '">').appendTo(form);
+            }
+        };
+
+        form.find(':input[name!=ChangedField]').change(function() {
+            mark_changed(jQuery(this).attr('name'));
+        });
+
+        var plainMessageBox  = form.find('.messagebox');
+        var messageBoxId = plainMessageBox.attr('id');
+        if (CKEDITOR.instances && CKEDITOR.instances[messageBoxId]) {
+            var richTextEditor = CKEDITOR.instances[messageBoxId];
+            if ( richTextEditor ) {
+                richTextEditor.on('instanceReady', function () {
+                    this.on('change', function () {
+                        mark_changed(plainMessageBox.attr('name'));
+                    });
+                });
+            }
+        }
+    });
+
+    // Make actions dropdown scrollable in case screen is too short
+    jQuery(window).resize(function() {
+        jQuery('#li-page-actions > ul').css('max-height', jQuery(window).height() - jQuery('#rt-header-container').height());
+    }).resize();
+
+    // Handle implicit form submissions like hitting Return/Enter on text inputs
+    jQuery('form[name=search-results-filter]').submit(filterSearchResults);
+    jQuery('a.permalink').click(function() {
+        var link = jQuery(this);
+        jQuery.get(
+            RT.Config.WebPath + "/Helpers/Permalink",
+            { Code: link.data('code'), URL: link.data('url') },
+            showModal
+        );
+        return false;
+    });
 });
 
+function filterSearchResults () {
+    var clauses = [];
+
+    var queue_clauses = [];
+    jQuery('.search-results-filter input[name=Queue]:checked').each(function() {
+        queue_clauses.push( 'Queue = ' + '"' + jQuery(this).val() + '"' );
+    });
+
+    if ( queue_clauses.length ) {
+        clauses.push( '( ' + queue_clauses.join( ' OR ' ) + ' )' );
+    }
+
+    var status_clauses = [];
+    jQuery('.search-results-filter input[name=Status]:checked').each(function() {
+        status_clauses.push('Status = ' + '"' + jQuery(this).val() + '"' );
+    });
+
+    if ( status_clauses.length ) {
+        clauses.push( '( ' + status_clauses.join( ' OR ' ) + ' )' );
+    }
+
+    var sla_clauses = [];
+    jQuery('.search-results-filter input[name=SLA]:checked').each(function() {
+        var value = jQuery(this).val();
+        if ( value == 'NULL' ) {
+            sla_clauses.push( 'SLA IS NULL' );
+        }
+        else {
+            sla_clauses.push( 'SLA = ' + '"' + value + '"' );
+        }
+    });
+
+    var type_clauses = [];
+    jQuery('.search-results-filter input[name=Type]:checked').each(function() {
+        type_clauses.push('Type = ' + '"' + jQuery(this).val() + '"' );
+    });
+
+    if ( type_clauses.length ) {
+        clauses.push( '( ' + type_clauses.join( ' OR ' ) + ' )' );
+    }
+
+    var subject = jQuery('.search-results-filter input[name=Subject]').val();
+    if ( subject && subject.match(/\S/) ) {
+        clauses.push( '( Subject LIKE "' + subject.replace(/(["\\])/g, "\\$1") + '" )' );
+    }
+
+    [ 'Requestors', 'Requestor', 'Cc', 'AdminCc', ].forEach( function(role) {
+        var value = jQuery('.search-results-filter input[name=' + role + ']').val();
+        if ( value && value.match(/\S/) ) {
+            clauses.push( role + '.EmailAddress = ' + "'" + value + "'" );
+        }
+    });
+
+    jQuery('.search-results-filter :input[name=Owner]').each(function() {
+        var value = jQuery(this).val();
+        if ( value && value.match(/\S/) ) {
+            clauses.push( 'Owner.Name = ' + '"' + value + '"' );
+        }
+    });
+
+    jQuery('.search-results-filter input[name^=CustomRole]').each(function() {
+        var role = jQuery(this).attr('name');
+        var value = jQuery(this).val();
+        if ( value && value.match(/\S/) ) {
+            clauses.push( role + '.EmailAddress = ' + '"' + value + '"' );
+        }
+    });
+
+    [ 'Creator', 'LastUpdatedBy' ].forEach( function(role) {
+        var value = jQuery('.search-results-filter input[name=' + role + ']').val();
+        if ( value && value.match(/\S/) ) {
+            var subs = [];
+            clauses.push( role + ' = "' + value + '"' );
+        }
+    });
+
+    [ 'id', 'Told', 'Starts', 'Started', 'Due', 'Resolved', 'Created', 'LastUpdated', 'Priority', 'InitialPriority', 'FinalPriority', 'TimeWorked', 'TimeEstimated', 'TimeLeft' ].forEach(function(type) {
+        var subs = [];
+        [ 'EqualTo', 'GreaterThan', 'LessThan' ].forEach( function(op) {
+            var value = jQuery('.search-results-filter :input[name=' + type + op + ']').val();
+            if ( value && value.match(/\S/) ) {
+                if ( value.match(/\D/) ) {
+                    value = "'" + value + "'";
+                }
+
+                if ( op == 'EqualTo' ) {
+                    subs.push( type + ' = ' + value  );
+                }
+                else if ( op == 'GreaterThan' ) {
+                    subs.push( type + ' > ' + value  );
+                }
+                else {
+                    subs.push( type + ' < ' + value  );
+                }
+            }
+        });
+        if ( subs.length ) {
+            clauses.push( '( ' + subs.join( ' AND ' ) + ' )' );
+        }
+    });
+
+    jQuery('.search-results-filter input[name^=CustomField]:not(:checkbox)').each(function() {
+        var name = jQuery(this).attr('name');
+        var value = jQuery(this).val();
+        if ( value && value.match(/\S/) ) {
+            clauses.push( "( '" + name + "'" + ' LIKE "' + value.replace(/(["\\])/g, "\\$1") + '" )' );
+        }
+    });
+
+    var cf_select = {};
+    jQuery('.search-results-filter input[name^=CustomField]:checkbox:checked').each(function() {
+        var name = jQuery(this).attr('name');
+        var value = jQuery(this).val();
+        if ( !cf_select[name] ) {
+            cf_select[name] = [];
+        }
+        cf_select[name].push(value);
+    });
+    jQuery.each(cf_select, function(name, values) {
+        var subs = [];
+        values.forEach(function(value) {
+            subs.push( "'" + name + "'" + ' = ' + '"' + value + '"' );
+        });
+        clauses.push( '( ' + subs.join( ' OR ' ) + ' )' );
+    });
+
+    var refresh_form = jQuery('div.refresh form');
+    var base_query = refresh_form.find('input[name=BaseQuery]').val();
+
+    var query;
+    if ( clauses.length ) {
+        if ( base_query.match(/^\s*\(.+\)\s*$/) ) {
+            query = base_query + " AND " + clauses.join( ' AND ' );
+        }
+        else {
+            query = '( ' + base_query + " ) AND " + clauses.join( ' AND ' );
+        }
+    }
+    else {
+        query = base_query;
+    }
+
+    refresh_form.find('input[name=Query]').val(query);
+    refresh_form.submit();
+    return false;
+};
+
 /* inline edit */
 jQuery(function () {
     var inlineEditEnabled = true;
@@ -926,7 +1203,6 @@ jQuery(function () {
 
     var cancelInlineEdit = function (editor) {
         var cell = editor.closest('td');
-        cell.find('[data-toggle=tooltip]').tooltip('hide');
 
         cell.removeClass('editing');
         editor.get(0).reset();
@@ -940,7 +1216,6 @@ jQuery(function () {
 
     var submitInlineEdit = function (editor) {
         var cell = editor.closest('td');
-        cell.find('[data-toggle=tooltip]').tooltip('hide');
 
         if (!inlineEditEnabled) {
             return;
@@ -1047,12 +1322,12 @@ jQuery(function () {
     });
 
     jQuery('.titlebox[data-inline-edit-behavior="click"] > .titlebox-content').click(function (e) {
-        if (jQuery(e.target).is('a, input, select, textarea')) {
+        if (jQuery(e.target).is('input, select, textarea')) {
             return;
         }
 
-        // Bypass radio/checkbox controls too
-        if (jQuery(e.target).closest('div.custom-radio, div.custom-checkbox').length) {
+        // Bypass links, buttons and radio/checkbox controls too
+        if (jQuery(e.target).closest('a, button, div.custom-radio, div.custom-checkbox').length) {
             return;
         }
 
@@ -1142,15 +1417,19 @@ jQuery(function() {
         selector: '[data-toggle=tooltip]',
         trigger: 'hover focus'
     });
+
+    // Hide the tooltip everywhere when the element is clicked
+    jQuery('[data-toggle="tooltip"]').click(function () {
+        jQuery('[data-toggle="tooltip"]').tooltip("hide");
+    });
 });
 
 // toggle bookmark for Ticket/Elements/Bookmark.
-// before replacing the bookmark content, hide then dispose of the existing tooltip to
+// before replacing the bookmark content, dispose of the existing tooltip to
 // ensure the tooltips are cycled correctly.
 function toggle_bookmark(url, id) {
     jQuery.get(url, function(data) {
         var bs_tooltip = jQuery('div[id^="tooltip"]');
-        bs_tooltip.tooltip('hide');
         bs_tooltip.tooltip('dispose');
         jQuery('.toggle-bookmark-' + id).replaceWith(data);
     });
@@ -1194,3 +1473,24 @@ function toggleTransactionDetails () {
 
     return false;
 }
+
+// Use Growl to show any UserMessages written to the page
+jQuery( function() {
+    var userMessages = RT.UserMessages;
+    for (var key in userMessages) {
+        jQuery.jGrowl(userMessages[key], { sticky: true, themeState: 'none' });
+    }
+} );
+
+function updateSelectpickerLiveSearch (element) {
+    element ||= jQuery('.selectpicker');
+    element.filter(':not([data-live-search])').each(function() {
+        jQuery(this).attr('data-live-search', jQuery(this).find('option').length >= RT.Config.SelectLiveSearchLimit ? true : false );
+    });
+}
+
+function refreshSelectpicker (element) {
+    element ||= jQuery('.selectpicker');
+    updateSelectpickerLiveSearch(element);
+    element.selectpicker('refresh');
+}
